local DEBUG = false

---------------------
-- Version
local RELEASETYPE = "Release"
local VERSION = GetAddOnMetadata("RaidWatch", "Version") or "1.0"
local REVISION = "1.46"

--[===[@alpha@
RELEASETYPE = "Alpha"
--@end-alpha@]===]

if REVISION:find("@") then 
	REVISION =  "634"
	RELEASETYPE = "Development"
	DEBUG = true
end

-- Globals
RWDEBUG = DEBUG
RWREVISION = REVISION
RWRELEASETPYE = RELEASETYPE
RWVERSION = VERSION

---------------------
-- Create Loader
RWL = LibStub("AceAddon-3.0"):NewAddon("RaidWatchLoD", "AceEvent-3.0", "AceConsole-3.0")
local RWL = RWL

---------------------
-- Locals
local L = LibStub("AceLocale-3.0"):GetLocale("RaidWatch2_Loader")

local LoadWithZone = {}
local LoadWithCore = {}

local EnabledZones = {}
local ModInfo = {}
local BossMods = {}

local InEnabledZone = false

---------------------
-- Init
function RWL:OnInitialize()
	local defaults = {
		profile = {
			minimap = {
				hide = false,
			},
			vicotorySound = "RW: Victory",
			GlobalBossHealthFrames = false,
			ShowFastestKillBar = true,
			
			-- Set if messages should use prefixes or not. i.e [RaidWatch] Enabled or [Yog-Saron] Malady on X
			UseCleanMessages = false,
			
			WindowButtonSize = 20,
			WindowButtonColor = {1, 1, 1, 1},
			WindowButtonHoverColor = {0.9, 0.5, 0.5},
			WindowButtonActiveColor = {1, 0.2, 0.5},
			WindowTitleColor = {0/255, 79/255, 173/255, 1},		
			WindowUseGradient = true,
			WindowTitleHeight = 30,
			
			WindowFontSettings = {
				Font = "accid",
				Size = 18,		
				Flags = "NONE",		
				Color = {1, 1, 1, 1},
			},
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("RaidWatch2DB", defaults, "Default")
	if RW then RW.db = self.db end	
	
	-- DB Upgrade
	for name, profile in pairs(self.db.profiles) do
		local db = profile
		db.WindowFontSettings = db.WindowFontSettings or {}
		if db.WindowFont then db.WindowFontSettings.Font = db.WindowFont end
		if db.WindowFontSize then db.WindowFontSettings.Size = db.WindowFontSize end
		if db.WindowFontFlags then db.WindowFontSettings.Flags = db.WindowFontFlags end
		if db.WindowFontColor then db.WindowFontSettings.Color = db.WindowFontColor end
		
		db.WindowFont = nil
		db.WindowFontSize = nil
		db.WindowFontFlags = nil
		db.WindowFontColor = nil	
	end	
	
	self.db.global.rev = REVISION
	
	for i = 1, GetNumAddOns() do
		if GetAddOnMetadata(i, "X-RW-Name") then
			local modId = GetAddOnInfo(i)
			local name 	= GetAddOnMetadata(i, "X-RW-Name") or ""
			local icon = GetAddOnMetadata(i, "X-RW-Icon")
			local order = GetAddOnMetadata(i, "X-RW-SortOrder")
			
			if GetAddOnMetadata(i, "X-RW-LoadWithZone") then
				local zones = {strsplit(",", GetAddOnMetadata(i, "X-RW-LoadWithZone") or "")}
				for i, zone in pairs(zones) do
					local zoneNumber = tonumber(zone)
					if not zoneNumber then zoneNumber = zone end
					LoadWithZone[zoneNumber] = LoadWithZone[zoneNumber] or {}					
					LoadWithZone[zoneNumber][modId] = {
						name = name,
						icon = icon,
					}
					EnabledZones[zoneNumber] = true
				end
			end
			
			if GetAddOnMetadata(i, "X-RW-BossMod") then
				BossMods[name] = true
			end
			
			if GetAddOnMetadata(i, "X-RW-LoadWithCore") then
				LoadWithCore[modId] = {
					name = name,
					icon = icon,
				}
			end
			
			ModInfo[name] = {
				modID = modId,
				icon = icon,
				order = order,
			}
		end
	end
end

---------------------
-- Enable
function RWL:OnEnable()
	if RELEASETYPE == "Alpha" then
		--self:Msg(L.ALPHA_INFO)
	elseif RELEASETYPE == "Development" then
		--self:Msg(L.DEV_INFO)
	end
	
	self:RegisterChatCommand("mrw", "SlashHandler")
	self:RegisterChatCommand("rw2", "SlashHandler")
	self:RegisterChatCommand("range", "ShowRange")
	self:RegisterChatCommand("rwtimerg", "ShowGlobalTimer")
	self:RegisterChatCommand("rwtimer", "ShowTimer")
	
	self:SetUpLDB()	
	
	if RW then
		RW.RegisterCallback(self, "CoreEnabled")
	end
	
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChange")
	-- Area ID is avilable at PEW event.
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "ZoneChange")
end

function RWL:Msg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(("|cff28579d[|r|cffcfe846%s|r|cff28579d]|r "):format(tostring(L.LOADER_NAME))..msg, 0.39, 0.67, 0.87)
end

-------------------------------------------
-- Events
function RWL:ZoneChange()	
	local areaID = GetCurrentMapAreaID()
	if LoadWithZone[areaID] then self:LoadCore(areaID) end
	if EnabledZones[areaID] then
		RW.Callbacks:Fire("EnteringEnabledZone", areaID)
		InEnabledZone = true
	elseif InEnabledZone then
		RW.Callbacks:Fire("LeavingEnabledZone")
		InEnabledZone = false
	end
end

-------------------------------------------
-- Messages

function RWL:CoreEnabled(...)
	for name, data in pairs(LoadWithCore) do
		self:LoadAddOn(name)
		LoadWithCore[name] = nil
	end
end

-------------------------------------------
-- Load functions
function RWL:LoadAddOn(addon)
	local loaded, errorMsg = LoadAddOn(addon)
	if not loaded then
		local name = select(2, GetAddOnInfo(addon))
		if not name then
			print("Error while loading "..addon..", it does not exist")
		else
			self:Msg(L.MOD_LOAD_ERROR:format(name, _G["ADDON_"..errorMsg]))
		end
		return
	end
	return true
end

function RWL:LoadCore(zone)
	if not RW then 
		self:LoadAddOn("RaidWatch_Core")
		if not RW then
			return 
		end
	end
	if not RW:IsEnabled() then
		RW:Enable()
		self.ldbojb.icon = [[Interface\Icons\Achievement_Boss_Sindragosa]]
	end
	if zone then
		self:LoadZone(zone)
	end
end

function RWL:LoadZone(zone)
	if LoadWithZone[zone] then
		for name, data in pairs(LoadWithZone[zone]) do
			if RWL:LoadAddOn(name) and BossMods[data.name] then
				self:Msg("Loaded "..data.name)
				RW.Callbacks:Fire("BossModLoaded", data.name)				
			end
			LoadWithZone[zone][name] = nil
			BossMods[data.name] = nil
		end
	end
end

function RWL:LoadByName(name)
	if ModInfo[name] then
		if self:LoadAddOn(ModInfo[name].modID) and BossMods[name] then
			self:Msg("Loaded "..name)
			RW.Callbacks:Fire("BossModLoaded", name)
		end
		for zone, data in pairs(LoadWithZone) do
			for mname, zonedata in pairs(data) do
				if mname == name then
					data[mname] = nil
				end
			end
		end
		BossMods[name] = nil
	end
end

function RWL:GetModInfo(name)
	return ModInfo[name]
end

function RWL:IterateBossMods()
	return pairs(BossMods)
end

--------------------------------------------
-- Slash commands
function RWL:SlashHandler(input, button, ...)
	if input == "" or input == "config" then
		if RW and button == "LeftButton" then
			RW.Callbacks:Fire("ToggleController")			
		end
		self:LoadCore()
		if RW then
			if not button or type(button) == "table" or button == "RightButton" then
				if not IsAddOnLoaded("RaidWatch_Options") and not RW:GetPlugin("Options") then
					if not self:LoadAddOn("RaidWatch_Options") then
						return
					end
				end
				local options = RW:GetPlugin("Options")
				if options then
					if  IsShiftKeyDown() then
						options:ToggleConfigMode()
					else					
						options:Enable()
						options:Toggle() 
					end
				end
			end			
		end
	end
end

function RWL:ShowRange(input)
	local number = tonumber(input)
	if type(number) == "number" then
		RW.Callbacks:Fire("RangeCheckShow", true, number, true)
	else
		RW.Callbacks:Fire("RangeCheckShow", true, nil, true)
	end
end

function RWL:ShowTimer(message)
	local time, text, id = string.split("*", message)
	if not (time and text) then
		self:Msg("Usage: /rwtimerg <time mm:ss> * <text> * [id]")
		return
	end
	local min, sec = string.split(":", time)
	if not sec and min then
		sec = min
		min = 0
	end
	time = (tonumber(min or "") or 0) * 60 + (tonumber(sec) or 0)
	
	self:LoadCore()
	RW.Callbacks:Fire("BarStart", id or math.random(), time, text)
end

function RWL:ShowGlobalTimer(message)
	local time, text, id = string.split("*", message)
	if not (time and text) then
		self:Msg("Usage: /rwtimerg <time mm:ss> * <text> * [id]")
		return
	end
	local min, sec = string.split(":", time)
	if not sec and min then
		sec = min
		min = 0
	end
	time = (tonumber(min or "") or 0) * 60 + (tonumber(sec) or 0)
	
	self:LoadCore()
	RW.Com:Send("StartCommBar", nil, nil, id or math.random(), time, text)
end

---------------------------------------------
-- LDB
function RWL:SetUpLDB()
	local LDB = LibStub("LibDataBroker-1.1")
	if not LDB then return end
	local l = LDB:NewDataObject("Raid Watch 2")
	l.type = "launcher"
	l.icon = [[Interface\Icons\Achievement_Boss_Sapphiron_01]]
	l.OnClick = function(self, button)
		RWL:SlashHandler("config", button)
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine(L.LDB_INFO)
		tt:AddLine(L.LDB_LEFTCLICK)
		tt:AddLine(L.LDB_RIGHTCLICK)
		tt:AddLine(L.LDB_SHIFTRIGHTCLICK)
	end
	local LibDBIcon = LibStub("LibDBIcon-1.0")
	if not LibDBIcon then return end
	LibDBIcon:Register("RW2", l, self.db.profile.minimap)
	
	self.ldbojb = l
end